@@ -25,16 +25,14 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public
 License along with libgfortran; see the file COPYING.  If not,
-write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+Boston, MA 02110-1301, USA.  */
 
 #include "config.h"
 #include "libgfortran.h"
 
 #include <errno.h>
 
-#include "../io/io.h"
-
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -43,7 +41,7 @@ Boston, MA 02111-1307, USA.  */
 # include <windows.h>
 # undef sleep
 # define sleep(x) Sleep(1000*(x))
-# define HAVE_SLEEP
+# define HAVE_SLEEP 1
 #endif
 
 /* SUBROUTINE SLEEP(SECONDS)
